# @Scenario     A_INTEL_LTE_PROTOCOM_KTCPRCV_0005.PY
#
# @Dependencies
# 1. 1 module with SIM card
#
# @Test Cases
# Test Name                                          Description
# A_INTEL_LTE_PROTOCOM_KTCPRCV_0005                  To check data can be received from HTTP server
#
# @History
# Date          Author         version             Modification
# 2015-11-05    Brliu          1.0                 Creation
# 2016-01-26    LLAW           1.1                 Update for HL7519/88
# 2016-03-18    LLAW           1.2                 migrate change and submit for 1-click run
# 2016-04-01    TYE            1.3              add support for HL7518
# 2016-04-06    Brliu          1.4                 Modify for HL7539/49
# 2016-04-13    tye            1.5                 update HL7518 behavior to follow HL8.
# 2016-06-02    tye            1.6                 add support for HL7690 to share code with HL7518.
# 2016-11-22    tqkieu         1.7                 cover some URCs when receive data from server
#-----------------------------------------------------------------------------------------------------

Ver_Script = "1.6"


def Setup_Module_kCnxCfg(COMn, GPRS_Cfg):
    #global MAX_CNX_CNT
    print "UART1: +KCNXCFG Connection Configuration..."

    for n in xrange(VarGlobal.iCnxStart, VarGlobal.iCnxStart + VarGlobal.iCnxTotal):
        SagSendAT(COMn, 'AT+KCNXCFG=%d,"GPRS","%s","%s","%s"\r' % (n, GPRS_Cfg[0], GPRS_Cfg[1], GPRS_Cfg[2]))
    if SagWaitnMatchResp(COMn, ["*\r\nOK\r\n"], 3000) != True:
        print "\nPROBLEM: +KCNXCFG command %d response is incorrect.\n" % n


def Setup_Module_kCgdcont(COMn, GPRS_Cfg):
   # global MAX_CNX_CNT
    print "UART1: +CGDCOND Connection Configuration..."
#TYE, add support for HL7518
    if (VarGlobal.NETWORK_ID in ['311480'] and VarGlobal.HardType_LTE in ['4','1']):
        pass
    elif (VarGlobal.NETWORK_ID in ['45406'] and VarGlobal.HardType_LTE in ['3']):
        pass
    else:
        SagSendAT(COMn, 'AT+CGATT?\r')
        SagWaitnMatchResp(COMn, ["*\r\nOK\r\n"], 5000)
        SagSendAT(COMn, 'AT+CGATT=0\r')
        SagWaitnMatchResp(COMn, ["\r\nOK\r\n"], 10000, update_result="not_critical")
        SagSendAT(COMn, 'AT+CGATT?\r')
        SagWaitnMatchResp(COMn, ["*\r\nOK\r\n"], 5000)



    SagSendAT(COMn, 'AT+CGDCONT?\r')
    SagWaitnMatchResp(COMn, ["*\r\nOK\r\n"], 5000)
    for n in xrange(VarGlobal.iCnxStart, VarGlobal.iCnxStart + VarGlobal.iCnxTotal):
        SagSendAT(COMn, 'AT+CGDCONT=%d,"IP","%s"\r' % (n, GPRS_Cfg[0]))
        if SagWaitnMatchResp(COMn, ["*\r\nOK\r\n"], 3000) != True:
            print "\nPROBLEM: +KCNXCFG command %d response is incorrect.\n" % n
    SagSendAT(COMn, 'AT+CGDCONT?\r')
    SagWaitnMatchResp(COMn, ["*\r\nOK\r\n"], 5000)


try:
    SIM_GPRS_LOGIN = get_ini_value(SIM_INI, 'gprs', 'Login')
except Exception, e:
    print e
    SIM_GPRS_LOGIN = ""

try:
    SIM_GPRS_PASSWORD = get_ini_value(SIM_INI, 'gprs', 'Password')
except Exception, e:
    print e
    SIM_GPRS_PASSWORD = ""


try:
    HardType_LTE = get_ini_value(HARD_INI, 'HardType_LTE', 'HardType_LTE')
    VarGlobal.HardType_LTE = get_ini_value(HARD_INI, 'HardType_LTE', 'HardType_LTE')
    VarGlobal.NETWORK_ID = get_ini_value(NETWORK_INI, 'Network', 'ID')

    SIM_Pin1 = get_ini_value(SIM_INI, 'Security', 'Pin1')
    VoiceNumber = get_ini_value(SIM_INI, 'Identification', 'VoiceNumber')

    if (VarGlobal.NETWORK_ID in ['311480']):
        SIM_GPRS_APN = get_ini_value(SIM_INI, 'gprs', 'APN_public')
    else:
        SIM_GPRS_APN = get_ini_value(SIM_INI, 'gprs', 'APN')

    print "SIM_GPRS_APN=", SIM_GPRS_APN
    print "SIM_GPRS_LOGIN=", SIM_GPRS_LOGIN
    print "SIM_GPRS_PASSWORD=", SIM_GPRS_PASSWORD

    GPRS_Cfg = (SIM_GPRS_APN, SIM_GPRS_LOGIN, SIM_GPRS_PASSWORD)

    VarGlobal.iCnxStart = 1
    VarGlobal.iCnxTotal = 1
    VarGlobal.lCnxToTest = [1]
#TYE, add support for HL7518
    if (VarGlobal.NETWORK_ID in ['311480'] and VarGlobal.HardType_LTE in ['4','1']):
        VarGlobal.iCnxStart = 5
        VarGlobal.iCnxTotal = 1
        VarGlobal.lCnxToTest = [5]
    elif (VarGlobal.NETWORK_ID in ['45406'] and VarGlobal.HardType_LTE in ['3']):
        #20160309, llaw, add support for HL7588 3G
        VarGlobal.iCnxStart = 5
        VarGlobal.iCnxTotal = 1
        VarGlobal.lCnxToTest = [5]
    else:
        pass

except Exception, e:
    print e
    print "\n---->Problem: exception happens when read parameter from initial files !!!\n\n"

###############################################################
print "\n----- Script Version = ", Ver_Script, " -----\n"

print "\n----- Program start -----\n"

UART1 = SagOpen(UART1_COM, baudrate)

# -------------------------- Module Initialization ----------------------------------

# Module Init
check_module(UART1, "Intel", ["AT ATE0 IPR"])
#init_module(UART1, "Intel", ["CFUN=1,1<60"])
#SagSleep(20000)  # especially for USB usage. Could be delete for true UART usage.
init_module(UART1, "Intel", ["AT&F ATE0 CMEE=1"])

check_module(UART1, "Intel", ["CGMR"])

#NOTE setup APN for module register on network
#20160309, llaw, add support for HL7588 3G
if (VarGlobal.HardType_LTE in ['3'] and VarGlobal.NETWORK_ID in ['45406']):
    #NOTE for testing HL7588 3G in HK SMT
    # set +HBHV=2,0
    # Setup_HBHV_2(UART1, 0)

    SagSendAT(UART1, "AT*PSRDBS=1,%d\r" %int(64+128))
    SagWaitnMatchResp(UART1, ["\r\nOK\r\n"], 3000)

    # set +CGDCONT=3,"IP","internet"

    SagSendAT(UART1, "AT+CGREG=2\r")
    SagWaitnMatchResp(UART1, ["*\r\nOK\r\n"], 3000)
    SagSendAT(UART1, "AT+CGREG?\r")
    SagWaitnMatchResp(UART1, ["\r\n+CGREG: 2,1,\"*\",\"*\",6,\"*\"\r\n"], 3000)
    SagWaitnMatchResp(UART1, ["*\r\nOK\r\n"], 3000)
    SagSendAT(UART1, "AT+CGREG=0\r")
    SagWaitnMatchResp(UART1, ["*\r\nOK\r\n"], 3000)

else:
    #NOTE for HL7519/88, HL7539
    wait_module(UART1, "Intel", ["+CEREG:1<5!24#"])


print "\n----- Testing Start -----\n"

print
print "Enable bi-directional hardware flow control..."
SagSendAT(UART1, "AT&K3\r")
SagWaitnMatchResp(UART1, ["\r\nOK\r\n"], 3000)

print
print "Check Current End code.."
SagSendAT(UART1, 'AT+KPATTERN?\r')
answer = SagWaitResp(UART1, ["\r\n+KPATTERN: *\r\n"], 3000)
result_END_PATTERN = SagMatchResp(answer, ["\r\n+KPATTERN: *\r\n"])
SagWaitnMatchResp(UART1, ["\r\nOK\r\n"], 3000)
if result_END_PATTERN:
    END_PATTERN = answer.split("\r\n")[1].split(" ")[1].split('"')[1]


VarGlobal.statOfItem = "OK"

MAX_ndata_available = 4294967295
# ------------------------------------------------------------------------------------
# Test Case A_INTEL_LTE_PROTOCOM_KTCPRCV_0005
# ------------------------------------------------------------------------------------
test_nb = ""
test_ID = "A_INTEL_LTE_PROTOCOM_KTCPRCV_0005"
PRINT_START_FUNC(test_nb + test_ID)

try:
    cnx_cnf = VarGlobal.lCnxToTest[0]

    print
    print "GPRS Connection Configuration..."
    Setup_Module_kCgdcont(UART1, GPRS_Cfg)
    Setup_Module_kCnxCfg(UART1, GPRS_Cfg)
    #tye, add support for HL7518 to follow HL8's behavior
    if (VarGlobal.HardType_LTE in ['1','10']):
        for HTTP_SERVER in ['"www.google.com"']: 
            print
            print "TCP Connection Configuration..."
            SagSendAT(UART1, 'AT+KTCPCFG=' + str(cnx_cnf) + ',0,' + HTTP_SERVER + ',80\r')
            answer = SagWaitResp(UART1, ["\r\n+KTCPCFG: *\r\n"], 3000)
            result = SagMatchResp(answer, ["\r\n+KTCPCFG: *\r\n"])
            SagWaitnMatchResp(UART1, ["*OK\r\n"], 3000)
               

            if result:
                session_id = answer.split("\r\n")[1].split(" ")[1]
                
                print "Start TCP Connection..."
                SagSendAT(UART1, 'AT+KTCPCNX=' + session_id + '\r')
                SagWaitnMatchResp(UART1, ["\r\nOK\r\n"], 3000)
                SagWaitnMatchResp(UART1, ["*\r\n+KTCP_IND: " + session_id + ",1\r\n"], 60000)

                DATA = "GET / HTTP/1.1\r\n\r\n" + chr(0x3)
                print
                print "Send data to HTTP server ..."
                SagSendAT(UART1, 'AT+KTCPSND=' + session_id + ',' + str(len(DATA)) + '\r')
                SagWaitnMatchResp(UART1, ["\r\nCONNECT\r\n"], 3000)

                SagSendAT(UART1,DATA)
                if result_END_PATTERN:
                    SagSendAT(UART1,END_PATTERN)
               
                
                print
                print "Check the received URC"
                answer = SagWaitResp(UART1, ["*\r\n+KTCP_DATA: " + session_id + ",*\r\n"], 30000)
                if SagMatchResp(answer, ["\r\n+KTCP_DATA: " + session_id + ",*\r\n"]):

                    print
                    print "The connection should be reset by HTTP server..."
  
                    SagWaitnMatchResp(UART1, ["*\r\n+KTCP_NOTIF: " + session_id + ",4\r\n"], 30000)  

                    print
                    print "Receive data with max number..."
                    SagSendAT(UART1, 'AT+KTCPRCV=' + session_id + ',' + str(MAX_ndata_available) + '\r')
                    SagWaitnMatchResp(UART1, ["\r\nCONNECT\r\n"], 3000)
                    SagWaitnMatchResp(UART1, ["*\r\nOK\r\n"], 60000)
                    
               

except Exception, e:
    print e
    VarGlobal.statOfItem = "NOK"
finally:
    pass


# Print test result
PRINT_TEST_RESULT(test_ID, VarGlobal.statOfItem)
# -----------------------------------------------------------------------------------


print "\n----- Testing End -----\n"

# Restore Module
SagSendAT(UART1, "AT&F\r")
SagWaitnMatchResp(UART1, ["*\r\nOK\r\n"], 3000)

# Close UART
SagClose(UART1)
